#!/bin/sh
#set -x

#######################################################################
# reset_perms.sh, ABr, 2003
#
# Script to reset file permissions for CVS.
# Because multiple users are accessing the CVS files,
# checkins or file adds can deny access permissions
# to other members of the CVS group. This post-processing
# script is called by CVS automatically whenever any
# file or directory is added or modified.
#
# The requirement is that all CVS users are in the same
# group, and that all CVS users can access files in the
# repository.
#
# Probably, all this script needs to do is to set the NT
# ACE for the affected file/directory. Historically, this
# script tried to solve the ACE access problems by using
# built-in Cygwin commands "chmod" and "chown". These
# don't seem to be necessary or effective, so in this script
# we simply identify the changed file(s) and call SetACL.exe
# explicitly.
#
# SetACL.exe is an open-source NT access control list
# command tool. More information on it is available at
# http://setacl.sourceforge.net/.
#
# Note that parse_cvs.pl must be available in order for this
# script to work.
#
# Developed by softwareAB
# http://softwareab.net/

#######################################################################
# CONFIGURATION SECTION

# Update this to point to where your CVS scripts are located.
# The files parse_cvs.pl and SetACL.exe must be located in this
# directory. Plus, the files must be read/executable by all.
CVS_SCRIPTS="/cvs_scripts"

# update this to point where you want log files to go
CVS_PROCESS="/cvs_process"

# by default, this script allows you to have three sets of permissions.
# ADMINS should be local administrators who
# CVSGROUP_1 should be the NT group for developers who will
#   have read/write repository access
# CVSGROUP_2 should be the NT group for developers who will
#   have read-only repository access
ADMINS=".\\Administrators"
CVSGROUP_1=".\\CVS Users"
CVSGROUP_2=".\\CVS Guests"

# set this variable to ONE (1) for full debugging
FULL_DEBUG="0"

# CONFIGURATION SECTION
#######################################################################

my_cvs_debug() {
  if [ -f "${_MY_CVSPROCESSLOG}" ]
  then
    # send to central file
    echo $* 2>&1 >>"${_MY_CVSPROCESSLOG}"
  fi

  # only if full debugging is on
  if [ "${FULL_DEBUG}" = "1" ]
  then
    echo $* 2>&1
  fi
}

set_cvs_ace() {
  _LOCAL_FNAME="$1"
  if [ -x ${SET_ACL} ]
  then
    _LOCAL_FNAME_DOS=`cygpath --dos --absolute "${_LOCAL_FNAME}"`
    my_cvs_debug "Setting ACL for ${_LOCAL_FNAME_DOS}"
    my_cvs_debug "Cmd Line: ${SET_ACL} -on \"${_LOCAL_FNAME_DOS}\" -ot file -actn ace -ace \"n:${CVSGROUP_1};p:Full\" -ace \"n:${CVSGROUP_2};p:Full\" -ace \"n:${ADMINS};p:Full\""
    if [ -f "${_MY_CVSPROCESSLOG}" ]
    then
      ${SET_ACL} -on "${_LOCAL_FNAME_DOS}" -ot file -actn ace -ace "n:${CVSGROUP_1};p:Full" -ace "n:${CVSGROUP_2};p:Full" -ace "n:${ADMINS};p:Full" 2>&1 >>"${_MY_CVSPROCESSLOG}"
    else
      ${SET_ACL} -on "${_LOCAL_FNAME_DOS}" -ot file -actn ace -ace "n:${CVSGROUP_1};p:Full" -ace "n:${CVSGROUP_2};p:Full" -ace "n:${ADMINS};p:Full" 2>&1 >/dev/null
    fi
  fi
}

set_process_log_ace() {
  set_cvs_ace "${_MY_CVSPROCESSLOG}"
}

process_fname() {
  # the dir first
  DIR=$1
  shift

  # empty starting name
  PREV_TOKENS=""

  # save the first token
  FIRST_TOKEN=""

  # the biggest valid fname found
  VALID_FNAME=""
  VALID_ACTUAL_FNAME=""
  VALID_ATTIC_FNAME=""
  VALID_ATTIC_ACTUAL_FNAME=""

  # process the file names
  while [ -n "$1" ]
  do
    # get the next token
    NEXT_TOKEN=$1
    #my_cvs_debug "NEXT_TOKEN='${NEXT_TOKEN}'"

    # save if necessary
    if [ -z "${FIRST_TOKEN}" ]
    then
      FIRST_TOKEN="${NEXT_TOKEN}"
    fi

    # put the tokens done so far together
    if [ -n "${PREV_TOKENS}" ]
    then
      ALL_TOKENS="${PREV_TOKENS} ${NEXT_TOKEN}"
    else
      ALL_TOKENS="${NEXT_TOKEN}"
    fi

    # build the candidate name
    CANDIDATE_FNAME="${DIR}/${ALL_TOKENS}"
    ATTIC_CANDIDATE_FNAME="${DIR}/Attic/${ALL_TOKENS}"
    ACTUAL_FNAME="${CANDIDATE_FNAME},v"
    ATTIC_ACTUAL_FNAME="${ATTIC_CANDIDATE_FNAME},v"
    #my_cvs_debug "ACTUAL_FNAME='${ACTUAL_FNAME}'"

    # if this is a filename, save as the biggest one found
    if [ -f "${ACTUAL_FNAME}" ]
    then
      VALID_FNAME="${ALL_TOKENS}"
      VALID_ACTUAL_FNAME="${ACTUAL_FNAME}"
      my_cvs_debug "VALID_FNAME='${VALID_FNAME}'"
    fi

    if [ -f "${ATTIC_ACTUAL_FNAME}" ]
    then
      VALID_ATTIC_FNAME="${ALL_TOKENS}"
      VALID_ATTIC_ACTUAL_FNAME="${ATTIC_ACTUAL_FNAME}"
      my_cvs_debug "VALID_ATTIC_ACTUAL_FNAME=${VALID_ATTIC_ACTUAL_FNAME}"
    fi

    # append to previous tokens read
    if [ -n "${PREV_TOKENS}" ]
    then
      PREV_TOKENS="${PREV_TOKENS} ${NEXT_TOKEN}"
    else
      PREV_TOKENS="${NEXT_TOKEN}"
    fi

    # next token in the list
    shift
  done

  # assume result is simply the first token we parsed
  RC_RESULT=""

  # change the file owner
  if [ -n "${VALID_ACTUAL_FNAME}" ]
  then
    my_cvs_debug "Updating attrs for ${VALID_ACTUAL_FNAME}"
    set_cvs_ace "${VALID_ACTUAL_FNAME}"
    RC_RESULT="${VALID_FNAME}"
  fi

  # check the Attic
  if [ -n "${VALID_ATTIC_ACTUAL_FNAME}" ]
  then
    my_cvs_debug "Updating attrs for ATTIC file ${VALID_ATTIC_ACTUAL_FNAME}"
    set_cvs_ace "${VALID_ATTIC_ACTUAL_FNAME}"
    RC_RESULT="${VALID_ATTIC_FNAME}"
  fi

  # check for error
  if [ -z "${RC_RESULT}" ]
  then
    my_cvs_debug "Could not locate a filename or attic name!"
    RC_RESULT="${FIRST_TOKEN}"
  fi

  # result is the biggest valid filename found
  echo ${RC_RESULT}
}

process_file_list() {
  _L_ACTION=$1
  _MY_WORK="${CVSROOT}/$$.wrk"

  # check for a file list
  my_cvs_debug "Checking for ${_L_ACTION} files..."
  FILES=`perl "${CVS_SCRIPTS}/parse_cvs.pl" "${_MY_WORK}" ${_L_ACTION}`
  if [ -n "${FILES}" ]
  then
    # one or more space-delimited entries
    my_cvs_debug "FILES(0)='${FILES}'"

    # 0. check for tag
    FILES=`echo ${FILES}|sed -e 's/Tag: [^ ]\+ \(.*\)/\1/' 2>/dev/null`
    my_cvs_debug "FILES(1)='${FILES}'"

    # 1. clear off leading/trailing spaces
    FILES=`echo ${FILES}|sed -e 's/[ \t]\*\(.*\)[ \t]\*/\1/' 2>/dev/null`
    my_cvs_debug "FILES(2)='${FILES}'"

    # 2. process file tokens
    PREV_FILES="${FILES}"
    while [ -n "${FILES}" ]
    do
      NEXT_FNAME=`process_fname "${DIR}" ${FILES}`
      FILES=`echo ${FILES}|sed -e "s/${NEXT_FNAME}\(.*\)/\1/" 2>/dev/null`
      my_cvs_debug "FILES(3)='${FILES}'"
      FILES=`echo ${FILES}|sed -e 's/[ \t]\*\(.*\)[ \t]\*/\1/' 2>/dev/null`
      my_cvs_debug "FILES(4)='${FILES}'"

      # if FILES is unchanged, then we have a problem and we must exit
      if [ "${FILES}" = "${PREV_FILES}" ]
      then
        FILES=""
      else
        PREV_FILES="${FILES}"
      fi
    done
  fi
}

# set the process log
PROCESS_LOG_DIR="${CVS_PROCESS}"
if [ ! -d "${PROCESS_LOG_DIR}" ]
then
  mkdir "${PROCESS_LOG_DIR}"
  chmod 777 "${PROCESS_LOG_DIR}"
fi
_MY_CVSPROCESSLOG=""
if [ -d "${PROCESS_LOG_DIR}" ]
then
  _MY_CVSPROCESSLOG="${PROCESS_LOG_DIR}/$$.log"
  touch "${_MY_CVSPROCESSLOG}"
fi

# SetACL location
SET_ACL=${CVS_SCRIPTS}/SetACL.exe

# read in the values passed from CVS
VALS=`cat`
my_cvs_debug "VALS(1)='${VALS}'"

# extract the directory
DIR=`echo ${VALS}|sed -e 's/Update of \(.*\) In directory .*/\1/g' 2>/dev/null`
my_cvs_debug "DIR='${DIR}'"

# is this from adding a directory?
IS_DIR_ADD=`echo ${VALS}|grep -e "Directory ${DIR} added to the repository" 2>/dev/null`
my_cvs_debug "IS_DIR_ADD='${IS_DIR_ADD}'"
if [ -n "${IS_DIR_ADD}" ]
then
  # simple, just set the permissions on the single directory (never multiple)
  my_cvs_debug "Updating attrs for ${VALID_ACTUAL_FNAME}"
  set_cvs_ace "${DIR}"
else
  # read over the cvs list
  _MY_WORK="${CVSROOT}/$$.wrk"
  echo ${VALS} > "${_MY_WORK}"
  process_file_list added
  process_file_list modified
  process_file_list removed
  rm -f "${_MY_WORK}"
fi

# output if desired
if [ -f "${_MY_CVSPROCESSLOG}" ]
then
  echo "See ${_MY_CVSPROCESSLOG} for post-processing details"

  # uncomment for full debugging
  if [ "${FULL_DEBUG}" = "1" ]
  then
    cat "${_MY_CVSPROCESSLOG}"
  fi
fi

# ensure that process log is set correctly
set_process_log_ace

# never return an error
exit 0

